/*-
catmem: concatenate (to standard output) named share memory regions.
Modeled roughly after the standard utility "cat"

Note the compile option "SHOULDWORK"
     -- I think the lseek(2) (or a stat(2)) call should work.
     Unfortunately, with the current release, it doesn't.

*/

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>



#define T_ACCESS "catmem: cannot access memory object %s (%s)\n"
#define T_NONAME "catmem: must supply filename\n"
#define T_BADARG "use: catmem file...\n"

main(int argc, char **argv)
{
	int	nerrs=0;
	int	c;

	while ((c=getopt(argc, argv, "")) != EOF) {
		fprintf(stderr,T_BADARG);

	}
	if (optind == argc) {
		fprintf(stderr,T_NONAME);
		exit(EXIT_FAILURE);
	}
	for (c=optind; c < argc; c++) {
		long	offs;
		char	*s;
		int	fd;
		if ((fd=shm_open(argv[optind],O_RDONLY,0)) == -1) {
			fprintf(stderr, T_ACCESS, argv[optind],strerror(errno));
			nerrs++;
			continue;
		}
#ifdef SHOULDWORK
		if ((offs = lseek(fd, 0, SEEK_END)) == -1) {
			perror("lseek");
			close(fd);
			continue;
		}
		if ((s=mmap(0,offs,PROT_READ, MAP_SHARED,fd,0)) == (caddr_t)-1) {
			perror("mmap");
			close(fd);
			continue;
		}
		write(1,s,offs);
		munmap(s,offs);
#else
		for (offs=0; ; offs += 4*1024) {
			if ((s=mmap(0,4*1024,PROT_READ,MAP_SHARED,fd,offs)) == (caddr_t)-1) {
				break;
			}
			write(1,s,4*1024);
			munmap(s,4*1024);
		}
#endif
		close(fd);
	}
	return nerrs ? EXIT_FAILURE : EXIT_SUCCESS;
}
